@@ -9,6 +9,7 @@ use syntect::highlighting::Theme as SyntaxTheme;
 use syntect::parsing::SyntaxSet;
 
 use crate::config::delta_unreachable;
+use crate::env::DeltaEnv;
 use crate::git_config::{GitConfig, GitConfigEntry};
 use crate::options;
 use crate::utils;
@@ -259,6 +260,16 @@ pub struct Opt {
     /// Format of `git blame` timestamp in raw git output received by delta.
     pub blame_timestamp_format: String,
 
+    #[clap(long = "blame-timestamp-output-format", value_name = "FMT")]
+    /// Format string for git blame timestamp output.
+    ///
+    /// This string is used for formatting the timestamps in git blame output. It must follow
+    /// the `strftime` format syntax specification. If it is not present, the timestamps will
+    /// be formatted in a human-friendly but possibly less accurate form.
+    ///
+    /// See: (https://docs.rs/chrono/latest/chrono/format/strftime/index.html)
+    pub blame_timestamp_output_format: Option<String>,
+
     #[clap(long = "color-only")]
     /// Do not alter the input structurally in any way.
     ///
@@ -328,9 +339,9 @@ pub struct Opt {
     ///
     /// A feature is a named collection of delta options in ~/.gitconfig. See FEATURES section. The
     /// environment variable DELTA_FEATURES can be set to a space-separated list of feature names.
-    /// If this is preceded with a space, the features from the environment variable will be added
+    /// If this is preceded with a + character, the features from the environment variable will be added
     /// to those specified in git config. E.g. DELTA_FEATURES=+side-by-side can be used to activate
-    /// side-by-side temporarily.
+    /// side-by-side temporarily (use DELTA_FEATURES=+ to go back to just the features from git config).
     pub features: Option<String>,
 
     #[clap(
@@ -510,7 +521,7 @@ pub struct Opt {
     pub hyperlinks: bool,
 
     #[clap(long = "hyperlinks-commit-link-format", value_name = "FMT")]
-    /// Format string for commit hyperlinks (requiraes --hyperlinks).
+    /// Format string for commit hyperlinks (requires --hyperlinks).
     ///
     /// The placeholder "{commit}" will be replaced by the commit hash. For example:
     /// --hyperlinks-commit-link-format='https://mygitrepo/{commit}/'
@@ -1008,8 +1019,8 @@ pub struct Opt {
     #[clap(long = "wrap-max-lines", default_value = "2", value_name = "N")]
     /// How often a line should be wrapped if it does not fit.
     ///
-    /// Zero means to never wrap. Any content which does not fit will be truncated. A value of
-    /// "unlimited" means a line will be wrapped as many times as required.
+    /// Zero means to never wrap. Any content which does not fit after wrapping will be truncated. A
+    /// value of "unlimited" means a line will be wrapped as many times as required.
     pub wrap_max_lines: String,
 
     #[clap(
@@ -1072,6 +1083,9 @@ pub struct Opt {
 
     #[clap(skip)]
     pub git_config_entries: HashMap<String, GitConfigEntry>,
+
+    #[clap(skip)]
+    pub env: DeltaEnv,
 }
 
 #[derive(Default, Clone, Debug)]
@@ -1088,7 +1102,7 @@ pub struct ComputedValues {
     pub true_color: bool,
 }
 
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub enum Width {
     Fixed(usize),
     Variable,
@@ -1100,7 +1114,7 @@ impl Default for Width {
     }
 }
 
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, PartialEq, Eq)]
 pub enum InspectRawLines {
     True,
     False,
@@ -1120,28 +1134,40 @@ impl Default for PagingMode {
 
 impl Opt {
     pub fn from_args_and_git_config(
+        env: DeltaEnv,
         git_config: Option<GitConfig>,
         assets: HighlightingAssets,
     ) -> Self {
-        Self::from_clap_and_git_config(Self::into_app().get_matches(), git_config, assets)
+        Self::from_clap_and_git_config(env, Self::into_app().get_matches(), git_config, assets)
     }
 
-    pub fn from_iter_and_git_config<I>(iter: I, git_config: Option<GitConfig>) -> Self
+    pub fn from_iter_and_git_config<I>(
+        env: DeltaEnv,
+        iter: I,
+        git_config: Option<GitConfig>,
+    ) -> Self
     where
         I: IntoIterator,
         I::Item: Into<OsString> + Clone,
     {
         let assets = utils::bat::assets::load_highlighting_assets();
-        Self::from_clap_and_git_config(Self::into_app().get_matches_from(iter), git_config, assets)
+        Self::from_clap_and_git_config(
+            env,
+            Self::into_app().get_matches_from(iter),
+            git_config,
+            assets,
+        )
     }
 
     fn from_clap_and_git_config(
+        env: DeltaEnv,
         arg_matches: clap::ArgMatches,
         mut git_config: Option<GitConfig>,
         assets: HighlightingAssets,
     ) -> Self {
         let mut opt = Opt::from_arg_matches(&arg_matches)
             .unwrap_or_else(|_| delta_unreachable("Opt::from_arg_matches failed"));
+        opt.env = env;
         options::set::set_options(&mut opt, &mut git_config, &arg_matches, assets);
         opt.git_config = git_config;
         opt
