@@ -40,6 +40,7 @@ pub struct Config {
     pub blame_palette: Vec<String>,
     pub blame_separator_style: Option<Style>,
     pub blame_timestamp_format: String,
+    pub blame_timestamp_output_format: Option<String>,
     pub color_only: bool,
     pub commit_regex: Regex,
     pub commit_style: Style,
@@ -240,6 +241,7 @@ impl From<cli::Opt> for Config {
             blame_separator_format: parse_blame_line_numbers(&opt.blame_separator_format),
             blame_separator_style: styles.remove("blame-separator-style"),
             blame_timestamp_format: opt.blame_timestamp_format,
+            blame_timestamp_output_format: opt.blame_timestamp_output_format,
             commit_style: styles["commit-style"],
             color_only: opt.color_only,
             commit_regex,
