@@ -26,10 +26,14 @@ pub fn format_commit_line_with_osc8_commit_hyperlink<'a>(
 ) -> Cow<'a, str> {
     if let Some(commit_link_format) = &config.hyperlinks_commit_link_format {
         COMMIT_LINE_REGEX.replace(line, |captures: &Captures| {
-            let commit = captures.get(2).unwrap().as_str();
-            format_osc8_hyperlink(&commit_link_format.replace("{commit}", commit), commit)
+            let prefix = captures.get(1).map(|m| m.as_str()).unwrap_or("");
+            let commit = captures.get(2).map(|m| m.as_str()).unwrap();
+            let suffix = captures.get(3).map(|m| m.as_str()).unwrap_or("");
+            let formatted_commit =
+                format_osc8_hyperlink(&commit_link_format.replace("{commit}", commit), commit);
+            format!("{}{}{}", prefix, formatted_commit, suffix)
         })
-    } else if let Some(GitConfigEntry::GitRemote(GitRemoteRepo::GitHubRepo(repo))) =
+    } else if let Some(GitConfigEntry::GitRemote(repo)) =
         config.git_config.as_ref().and_then(get_remote_url)
     {
         COMMIT_LINE_REGEX.replace(line, |captures: &Captures| {
@@ -93,12 +97,12 @@ lazy_static! {
 
 fn format_commit_line_captures_with_osc8_commit_hyperlink(
     captures: &Captures,
-    github_repo: &str,
+    repo: &GitRemoteRepo,
 ) -> String {
     let commit = captures.get(2).unwrap().as_str();
     format!(
         "{prefix}{osc}8;;{url}{st}{commit}{osc}8;;{st}{suffix}",
-        url = format_github_commit_url(commit, github_repo),
+        url = repo.format_commit_url(commit),
         commit = commit,
         prefix = captures.get(1).map(|m| m.as_str()).unwrap_or(""),
         suffix = captures.get(3).unwrap().as_str(),
@@ -107,10 +111,6 @@ fn format_commit_line_captures_with_osc8_commit_hyperlink(
     )
 }
 
-fn format_github_commit_url(commit: &str, github_repo: &str) -> String {
-    format!("https://github.com/{}/commit/{}", github_repo, commit)
-}
-
 #[cfg(not(target_os = "windows"))]
 #[cfg(test)]
 pub mod tests {
