@@ -2,14 +2,12 @@ mod git_config_entry;
 
 pub use git_config_entry::{GitConfigEntry, GitRemoteRepo};
 
+use crate::env::DeltaEnv;
 use regex::Regex;
 use std::collections::HashMap;
-use std::env;
 #[cfg(test)]
 use std::path::Path;
 
-use crate::fatal;
-
 use lazy_static::lazy_static;
 
 pub struct GitConfig {
@@ -17,12 +15,33 @@ pub struct GitConfig {
     config_from_env_var: HashMap<String, String>,
     pub enabled: bool,
     pub repo: Option<git2::Repository>,
+    // To make GitConfig cloneable when testing (in turn to make Config cloneable):
+    #[cfg(test)]
+    path: std::path::PathBuf,
+}
+
+#[cfg(test)]
+impl Clone for GitConfig {
+    fn clone(&self) -> Self {
+        assert!(self.repo.is_none());
+        GitConfig {
+            // Assumes no test modifies the file pointed to by `path`
+            config: git2::Config::open(&self.path).unwrap(),
+            config_from_env_var: self.config_from_env_var.clone(),
+            enabled: self.enabled,
+            repo: None,
+            path: self.path.clone(),
+        }
+    }
 }
 
 impl GitConfig {
-    pub fn try_create() -> Option<Self> {
-        let repo = match std::env::current_dir() {
-            Ok(dir) => git2::Repository::discover(dir).ok(),
+    #[cfg(not(test))]
+    pub fn try_create(env: &DeltaEnv) -> Option<Self> {
+        use crate::fatal;
+
+        let repo = match &env.current_dir {
+            Some(dir) => git2::Repository::discover(dir).ok(),
             _ => None,
         };
         let config = match &repo {
@@ -36,7 +55,7 @@ impl GitConfig {
                 });
                 Some(Self {
                     config,
-                    config_from_env_var: parse_config_from_env_var(),
+                    config_from_env_var: parse_config_from_env_var(env),
                     repo,
                     enabled: true,
                 })
@@ -46,16 +65,22 @@ impl GitConfig {
     }
 
     #[cfg(test)]
-    pub fn from_path(path: &Path, honor_env_var: bool) -> Self {
+    pub fn try_create(_env: &DeltaEnv) -> Option<Self> {
+        unreachable!("GitConfig::try_create() is not available when testing");
+    }
+
+    #[cfg(test)]
+    pub fn from_path(env: &DeltaEnv, path: &Path, honor_env_var: bool) -> Self {
         Self {
             config: git2::Config::open(path).unwrap(),
             config_from_env_var: if honor_env_var {
-                parse_config_from_env_var()
+                parse_config_from_env_var(env)
             } else {
                 HashMap::new()
             },
             repo: None,
             enabled: true,
+            path: path.into(),
         }
     }
 
@@ -71,9 +96,9 @@ impl GitConfig {
     }
 }
 
-fn parse_config_from_env_var() -> HashMap<String, String> {
-    if let Ok(s) = env::var("GIT_CONFIG_PARAMETERS") {
-        parse_config_from_env_var_value(&s)
+fn parse_config_from_env_var(env: &DeltaEnv) -> HashMap<String, String> {
+    if let Some(s) = &env.git_config_parameters {
+        parse_config_from_env_var_value(s)
     } else {
         HashMap::new()
     }
