@@ -2,70 +2,57 @@
 mod tests {
     use crate::ansi::{self, strip_ansi_codes};
     use crate::cli::InspectRawLines;
-    use crate::delta::State;
+    use crate::delta::{DiffType, State};
+    use crate::handlers::hunk_header::ParsedHunkHeader;
     use crate::style;
     use crate::tests::ansi_test_utils::ansi_test_utils;
     use crate::tests::integration_test_utils;
-    use crate::tests::test_utils;
+    use crate::tests::integration_test_utils::DeltaTest;
 
     #[test]
     fn test_added_file() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(ADDED_FILE_INPUT, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(output.contains("\nadded: a.py\n"));
+        DeltaTest::with_args(&[])
+            .with_input(ADDED_FILE_INPUT)
+            .expect_contains("\nadded: a.py\n");
     }
 
     #[test]
-    #[ignore] // #128
     fn test_added_empty_file() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(ADDED_EMPTY_FILE, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(output.contains("\nadded: file\n"));
+        DeltaTest::with_args(&[])
+            .with_input(ADDED_EMPTY_FILE)
+            .expect_contains("\nadded: file\n");
     }
 
     #[test]
     fn test_added_file_directory_path_containing_space() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output =
-            integration_test_utils::run_delta(ADDED_FILES_DIRECTORY_PATH_CONTAINING_SPACE, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(output.contains("\nadded: with space/file1\n"));
-        assert!(output.contains("\nadded: nospace/file2\n"));
+        DeltaTest::with_args(&[])
+            .with_input(ADDED_FILES_DIRECTORY_PATH_CONTAINING_SPACE)
+            .expect_contains("\nadded: with space/file1\n")
+            .expect_contains("\nadded: nospace/file2\n");
     }
 
     #[test]
     fn test_renamed_file() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(RENAMED_FILE_INPUT, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(test_utils::contains_once(
-            &output,
-            "\nrenamed: a.py ⟶   b.py\n"
-        ));
+        DeltaTest::with_args(&[])
+            .with_input(RENAMED_FILE_INPUT)
+            .expect_contains_once("\nrenamed: a.py ⟶   b.py\n");
     }
 
     #[test]
     fn test_copied_file() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(GIT_DIFF_WITH_COPIED_FILE, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(test_utils::contains_once(
-            &output,
-            "\ncopied: first_file ⟶   copied_file\n"
-        ));
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_WITH_COPIED_FILE)
+            .expect_contains_once("\ncopied: first_file ⟶   copied_file\n");
     }
 
     #[test]
     fn test_renamed_file_with_changes() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(RENAMED_FILE_WITH_CHANGES_INPUT, &config);
-        let output = strip_ansi_codes(&output);
-        println!("{}", output);
-        assert!(test_utils::contains_once(
-            &output,
-            "\nrenamed: Casks/font-dejavusansmono-nerd-font.rb ⟶   Casks/font-dejavu-sans-mono-nerd-font.rb\n"));
+        let t = DeltaTest::with_args(&[])
+            .with_input(RENAMED_FILE_WITH_CHANGES_INPUT)
+            .expect_contains_once(
+            "\nrenamed: Casks/font-dejavusansmono-nerd-font.rb ⟶   Casks/font-dejavu-sans-mono-nerd-font.rb\n"
+            );
+        println!("{}", t.output);
     }
 
     #[test]
@@ -138,7 +125,7 @@ mod tests {
         assert_eq!(lines.nth(7).unwrap(), "This is different from b");
         // File uniqueness
         assert_eq!(lines.nth(2).unwrap(), "Only in a/: just_a");
-        // FileMeta divider
+        // DiffHeader divider
         assert!(lines.next().unwrap().starts_with("───────"));
         // Next hunk
         assert_eq!(
@@ -147,6 +134,15 @@ mod tests {
         );
     }
 
+    #[test]
+    fn test_diff_unified_concatenated() {
+        // See #1002. Line buffers were not being flushed correctly, leading to material from first
+        // file last hunk appearing at beginning of second file first hunk.
+        DeltaTest::with_args(&[])
+            .with_input(DIFF_UNIFIED_CONCATENATED)
+            .expect_contains("\nLINES.\n\n1/y 2022-03-06");
+    }
+
     #[test]
     #[ignore] // Ideally, delta would make this test pass. See #121.
     fn test_delta_ignores_non_diff_input() {
@@ -187,9 +183,7 @@ mod tests {
     fn test_diff_with_merge_conflict_is_not_truncated() {
         let config = integration_test_utils::make_config_from_args(&[]);
         let output = integration_test_utils::run_delta(DIFF_WITH_MERGE_CONFLICT, &config);
-        // TODO: The + in the first column is being removed.
-        assert!(strip_ansi_codes(&output).contains("+>>>>>>> Stashed changes"));
-        assert_eq!(output.lines().count(), 45);
+        println!("{}", strip_ansi_codes(&output));
     }
 
     #[test]
@@ -401,11 +395,6 @@ commit 94907c0f136f46dc46ffae2dc92dca9af7eb7c2e
         ]);
     }
 
-    #[test]
-    fn test_commit_style_box_ul_deprecated_options() {
-        _do_test_commit_style_box_ul(&["--commit-color", "blue", "--commit-style", "box"]);
-    }
-
     fn _do_test_commit_style_box(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -541,16 +530,6 @@ commit 94907c0f136f46dc46ffae2dc92dca9af7eb7c2e │
         ]);
     }
 
-    #[test]
-    fn test_commit_style_underline_deprecated_options() {
-        _do_test_commit_style_underline(&[
-            "--commit-color",
-            "yellow",
-            "--commit-style",
-            "underline",
-        ]);
-    }
-
     fn _do_test_commit_style_underline(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -718,11 +697,6 @@ src/align.rs
         ]);
     }
 
-    #[test]
-    fn test_file_style_box_ul_deprecated_options() {
-        _do_test_file_style_box_ul(&["--file-color", "green", "--file-style", "box"]);
-    }
-
     fn _do_test_file_style_box(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -798,11 +772,6 @@ src/align.rs │
         ]);
     }
 
-    #[test]
-    fn test_file_style_underline_deprecated_options() {
-        _do_test_file_style_underline(&["--file-color", "magenta", "--file-style", "underline"]);
-    }
-
     fn _do_test_file_style_underline(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -1221,11 +1190,6 @@ impl<'a> Alignment<'a> {
         ]);
     }
 
-    #[test]
-    fn test_hunk_header_style_box_deprecated_options() {
-        _do_test_hunk_header_style_box(&["--hunk-color", "white", "--hunk-style", "box"]);
-    }
-
     fn _do_test_hunk_header_style_box(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -1338,16 +1302,6 @@ src/align.rs:71: impl<'a> Alignment<'a> { │
         ]);
     }
 
-    #[test]
-    fn test_hunk_header_style_underline_deprecated_options() {
-        _do_test_hunk_header_style_underline(&[
-            "--hunk-color",
-            "black",
-            "--hunk-style",
-            "underline",
-        ]);
-    }
-
     fn _do_test_hunk_header_style_underline(args: &[&str]) {
         let config = integration_test_utils::make_config_from_args(args);
         let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
@@ -1384,7 +1338,12 @@ src/align.rs:71: impl<'a> Alignment<'a> { │
             4,
             "impl<'a> Alignment<'a> { ",
             "rs",
-            State::HunkHeader("".to_owned(), "".to_owned()),
+            State::HunkHeader(
+                DiffType::Unified,
+                ParsedHunkHeader::default(),
+                "".to_owned(),
+                "".to_owned(),
+            ),
             &config,
         );
         ansi_test_utils::assert_line_has_no_color(&output, 12, "─────────────────────────────┘");
@@ -1539,7 +1498,7 @@ src/align.rs:71: impl<'a> Alignment<'a> { │
             1,
             "        for (i, x_i) in self.x.iter().enumerate() {",
             "rs",
-            State::HunkZero,
+            State::HunkZero(DiffType::Unified, None),
             &config,
         );
     }
@@ -1581,51 +1540,97 @@ src/align.rs:71: impl<'a> Alignment<'a> { │
     }
 
     #[test]
-    fn test_file_mode_change_gain_executable_bit() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(
-            GIT_DIFF_FILE_MODE_CHANGE_GAIN_EXECUTABLE_BIT,
-            &config,
-        );
+    fn test_file_mode_change_with_rename() {
+        let config = integration_test_utils::make_config_from_args(&["--right-arrow=->"]);
+        let output =
+            integration_test_utils::run_delta(GIT_DIFF_FILE_MODE_CHANGE_WITH_RENAME, &config);
         let output = strip_ansi_codes(&output);
-        assert!(output.contains(r"src/delta.rs: mode +x"));
+        assert!(output.contains("renamed: old-longer-name -> shorter-name (mode +x)"));
+    }
+
+    #[test]
+    fn test_file_mode_change_gain_executable_bit() {
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_FILE_MODE_CHANGE_GAIN_EXECUTABLE_BIT)
+            .expect_contains("src/delta.rs (mode +x)");
     }
 
     #[test]
     fn test_file_mode_change_lose_executable_bit() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output = integration_test_utils::run_delta(
-            GIT_DIFF_FILE_MODE_CHANGE_LOSE_EXECUTABLE_BIT,
-            &config,
-        );
-        let output = strip_ansi_codes(&output);
-        assert!(output.contains(r"src/delta.rs: mode -x"));
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_FILE_MODE_CHANGE_LOSE_EXECUTABLE_BIT)
+            .expect_contains("src/delta.rs (mode -x)");
+    }
+
+    #[test]
+    fn test_file_mode_change_unexpected_bits() {
+        DeltaTest::with_args(&["--navigate", "--right-arrow=->"])
+            .with_input(GIT_DIFF_FILE_MODE_CHANGE_UNEXPECTED_BITS)
+            .expect_contains("Δ src/delta.rs (mode 100700 -> 100644)");
+    }
+
+    #[test]
+    fn test_file_deleted_without_preimage() {
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_FILE_DELETED_WITHOUT_PREIMAGE)
+            .expect_contains("removed: foo.bar");
+    }
+
+    #[test]
+    fn test_files_deleted_without_preimage() {
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_FILES_DELETED_WITHOUT_PREIMAGE)
+            .expect_contains("removed: foo")
+            .expect_contains("removed: bar");
+    }
+
+    #[test]
+    fn test_file_mode_change_with_diff() {
+        DeltaTest::with_args(&["--navigate", "--keep-plus-minus-markers"])
+            .with_input(GIT_DIFF_FILE_MODE_CHANGE_WITH_DIFF)
+            .expect_contains("Δ src/script (mode +x)")
+            .expect_after_skip(
+                5,
+                "
+                ─────┐
+                • 1: │
+                ─────┘
+                -#!/bin/sh
+                +#!/bin/bash",
+            );
     }
 
     #[test]
     fn test_hyperlinks_commit_link_format() {
-        let config = integration_test_utils::make_config_from_args(&[
-            // If commit-style is not set then the commit line is handled in raw
-            // mode, in which case we only format hyperlinks if output is a tty;
-            // this causes the test to fail on Github Actions, but pass locally
-            // if output is left going to the screen.
-            "--commit-style",
-            "blue",
-            "--hyperlinks",
-            "--hyperlinks-commit-link-format",
-            "https://invent.kde.org/utilities/konsole/-/commit/{commit}",
-        ]);
-        let output = integration_test_utils::run_delta(GIT_DIFF_SINGLE_HUNK, &config);
-        assert!(output.contains(r"https://invent.kde.org/utilities/konsole/-/commit/94907c0f136f46dc46ffae2dc92dca9af7eb7c2e"));
+        // If commit-style is not set then the commit line is handled in raw
+        // mode, in which case we only format hyperlinks if output is a tty;
+        // this causes the test to fail on Github Actions, but pass locally
+        // if output is left going to the screen.
+        DeltaTest::with_args(&[
+                "--commit-style",
+                "blue",
+                "--hyperlinks",
+                "--hyperlinks-commit-link-format",
+                "https://invent.kde.org/utilities/konsole/-/commit/{commit}",
+            ])
+            .with_input(GIT_DIFF_SINGLE_HUNK)
+            .expect_raw_contains(
+                r"https://invent.kde.org/utilities/konsole/-/commit/94907c0f136f46dc46ffae2dc92dca9af7eb7c2e"
+                );
     }
 
     #[test]
     fn test_filenames_with_spaces() {
-        let config = integration_test_utils::make_config_from_args(&[]);
-        let output =
-            integration_test_utils::run_delta(GIT_DIFF_NO_INDEX_FILENAMES_WITH_SPACES, &config);
-        let output = strip_ansi_codes(&output);
-        assert!(output.contains("a b ⟶   c d\n"));
+        DeltaTest::with_args(&[])
+            .with_input(GIT_DIFF_NO_INDEX_FILENAMES_WITH_SPACES)
+            .expect_contains("a b ⟶   c d\n");
+    }
+
+    #[test]
+    fn test_file_removal_in_log_output() {
+        DeltaTest::with_args(&[])
+            .with_input(GIT_LOG_FILE_REMOVAL_IN_FIRST_COMMIT)
+            .expect_after_header("#partial\n\nremoved: a");
     }
 
     const GIT_DIFF_SINGLE_HUNK: &str = "\
@@ -1900,6 +1905,27 @@ Only in b/: just_b
 +This is different from a
 ";
 
+    const DIFF_UNIFIED_CONCATENATED: &str = "\
+--- 1/x 2022-03-06 11:16:06.313403500 -0800
++++ 2/x 2022-03-06 11:18:14.083403500 -0800
+@@ -1,5 +1,5 @@
+ This
+-is
++IS
+ a
+ few
+-lines.
++LINES.
+--- 1/y 2022-03-06 11:16:44.483403500 -0800
++++ 2/y 2022-03-06 11:16:55.213403500 -0800
+@@ -1,4 +1,4 @@
+ This
+ is
+-another
++ANOTHER
+ test.
+";
+
     const NOT_A_DIFF_OUTPUT: &str = "\
 Hello world
 This is a regular file that contains:
@@ -2302,6 +2328,15 @@ Date:   Sun Nov 1 15:28:53 2020 -0500
 [32m+[m[32m][m
 "#;
 
+    const GIT_DIFF_FILE_MODE_CHANGE_WITH_RENAME: &str = "
+diff --git a/old-longer-name b/shorter-name
+old mode 100644
+new mode 100755
+similarity index 100%
+rename from old-longer-name
+rename to shorter-name
+";
+
     const GIT_DIFF_FILE_MODE_CHANGE_GAIN_EXECUTABLE_BIT: &str = "
 diff --git a/src/delta.rs b/src/delta.rs
 old mode 100644
@@ -2312,6 +2347,41 @@ new mode 100755
 diff --git a/src/delta.rs b/src/delta.rs
 old mode 100755
 new mode 100644
+";
+
+    const GIT_DIFF_FILE_MODE_CHANGE_UNEXPECTED_BITS: &str = "
+diff --git a/src/delta.rs b/src/delta.rs
+old mode 100700
+new mode 100644
+";
+
+    // This output can be generated with `git diff -D`
+    const GIT_DIFF_FILE_DELETED_WITHOUT_PREIMAGE: &str = "
+diff --git a/foo.bar b/foo.bar
+deleted file mode 100644
+index e019be0..0000000
+";
+
+    // This output can be generated with `git diff -D`
+    const GIT_DIFF_FILES_DELETED_WITHOUT_PREIMAGE: &str = "
+diff --git a/foo b/foo
+deleted file mode 100644
+index e019be0..0000000
+diff --git a/bar b/bar
+deleted file mode 100644
+index e019be0..0000000
+";
+
+    const GIT_DIFF_FILE_MODE_CHANGE_WITH_DIFF: &str = "
+diff --git a/src/script b/src/script
+old mode 100644
+new mode 100755
+index d00491f..0cfbf08 100644
+--- a/src/script
++++ b/src/script
+@@ -1 +1 @@
+-#!/bin/sh
++#!/bin/bash
 ";
 
     const GIT_DIFF_NO_INDEX_FILENAMES_WITH_SPACES: &str = "
@@ -2322,5 +2392,27 @@ index d00491f..0cfbf08 100644
 @@ -1 +1 @@
 -1
 +2
+";
+
+    const GIT_LOG_FILE_REMOVAL_IN_FIRST_COMMIT: &str = "
+commit 4117f616160180c0c57ea64840eadd08b7fa32a4
+Author: Björn Steinbrink <bsteinbr@gmail.com>
+Date:   Tue Jun 21 14:51:59 2022 +0200
+
+    remove file
+
+diff --git a a
+deleted file mode 100644
+index e69de29..0000000
+
+commit 190cce5dffeb9050fd6a27780f16d84b19c07dc0
+Author: Björn Steinbrink <bsteinbr@gmail.com>
+Date:   Tue Jun 21 14:48:20 2022 +0200
+
+    add file
+
+diff --git a a
+new file mode 100644
+index 0000000..e69de29
 ";
 }
